import codecs
import json
import jsonschema
import requests
import six

from kinto_client import cli_utils
from six.moves.urllib.parse import urljoin

from . import constants
from .amo import prepare_amo_records
from .kinto import get_kinto_records
from .logger import logger
from .synchronize import get_diff, push_changes


FIELDS = {
    'addons': ['blockID', 'os', 'guid', 'prefs', 'versionRange', 'details'],
    'certificates': ['serialNumber', 'issuerName', 'details'],
    'gfx': ['blockID', 'os', 'vendor', 'feature', 'featureStatus',
            'driverVersion', 'driverVersionmax', 'driverVersionComparator',
            'devices', 'details'],
    'plugins': ['blockID', 'os', 'xpcomabi', 'infoURL', 'versionRange',
                'matchName', 'matchFilename', 'matchDescription', 'details']
}


def sync_records(amo_records, fields,
                 kinto_client, bucket, collection, config, permissions):

    amo_records = prepare_amo_records(amo_records, fields)

    if config:
        # We validate all amo_records to make sure nothing is broken
        # if we were to import everything.
        #
        # We could be validating only to_create records below but it
        # is a sane safety check.
        #
        for record in amo_records:
            jsonschema.validate(record, config['schema'])

    kinto_records = get_kinto_records(
        kinto_client=kinto_client,
        bucket=bucket,
        collection=collection,
        config=config,
        permissions=permissions)

    to_create, to_update, to_delete = get_diff(amo_records, kinto_records)

    push_changes((to_create, to_update, to_delete), kinto_client,
                 bucket=bucket, collection=collection)


def main(args=None):
    parser = cli_utils.add_parser_options(
        description='Import the blocklists from the addons server into Kinto.',
        default_collection=None,
        default_bucket=None,
        default_server=constants.KINTO_SERVER,
        default_auth=constants.AUTH,
        include_bucket=False,
        include_collection=False)

    parser.add_argument('-S', '--schema-file', help='JSON Schemas file',
                        type=str, default=constants.SCHEMA_FILE)

    parser.add_argument('--no-schema', help='Should we handle schemas',
                        action="store_true")

    parser.add_argument('--certificates-bucket',
                        help='Bucket name for certificates',
                        type=str, default=constants.CERT_BUCKET)

    parser.add_argument('--certificates-collection',
                        help='Collection name for certificates',
                        type=str, default=constants.CERT_COLLECTION)

    parser.add_argument('--gfx-bucket', help='Bucket name for gfx',
                        type=str, default=constants.GFX_BUCKET)

    parser.add_argument('--gfx-collection',
                        help='Collection name for gfx',
                        type=str, default=constants.GFX_COLLECTION)

    parser.add_argument('--addons-bucket', help='Bucket name for addons',
                        type=str, default=constants.ADDONS_BUCKET)

    parser.add_argument('--addons-collection',
                        help='Collection name for addon',
                        type=str, default=constants.ADDONS_COLLECTION)

    parser.add_argument('--plugins-bucket', help='Bucket name for plugins',
                        type=str, default=constants.PLUGINS_BUCKET)

    parser.add_argument('--plugins-collection',
                        help='Collection name for plugin',
                        type=str, default=constants.PLUGINS_COLLECTION)

    parser.add_argument('-C', '--certificates',
                        help='Only import certificates',
                        action='store_true')

    parser.add_argument('-G', '--gfx', help='Only import GFX drivers',
                        action='store_true')

    parser.add_argument('-A', '--addons', help='Only import addons',
                        action='store_true')

    parser.add_argument('-P', '--plugins', help='Only import plugins',
                        action='store_true')

    # Addons Server selection
    parser.add_argument('--addons-server',
                        help='The addons server to import from',
                        type=str, default=constants.ADDONS_SERVER)

    args = parser.parse_args(args=args)
    cli_utils.setup_logger(logger, args)

    # If none of the different "collections" were passed as parameter, then we
    # want to import them all.
    import_all = not any([
        args.certificates,
        args.gfx,
        args.addons,
        args.plugins])

    kinto_client = cli_utils.create_client_from_args(args)

    # Check if the schema capability is activated
    body, headers = kinto_client.session.request('get', '/')
    if 'schema' not in body.get('capabilities', {}):
        logger.warn('\t --- Server schema validation disabled --- \n')
        logger.warn("The server at {} won't validate the records against "
                    "the collection JSON schema. More information "
                    "http://kinto.readthedocs.io/en/stable/api/1.x/"
                    "collections.html?highlight=json%20validation"
                    "#collection-json-schema\n".format(args.server))

    # Load the schemas
    schemas = {}
    if not args.no_schema:
        with codecs.open(args.schema_file, 'r', encoding='utf-8') as f:
            schemas = json.load(f)['collections']

    blocklists_url = urljoin(args.addons_server, '/blocked/blocklists.json')
    resp = requests.get(blocklists_url)
    resp.raise_for_status()
    blocklists = resp.json()

    for collection_type, records in six.iteritems(blocklists):
        collection_type = collection_type.replace('-', '')
        if hasattr(args, collection_type) and (
                getattr(args, collection_type) or import_all):
            bucket = getattr(args, '%s_bucket' % collection_type)
            collection = getattr(args, '%s_collection' % collection_type)
            config = None
            if collection_type in schemas:
                config = schemas[collection_type]['config']
            sync_records(amo_records=records,
                         fields=FIELDS[collection_type],
                         kinto_client=kinto_client,
                         bucket=bucket,
                         collection=collection,
                         config=config,
                         permissions=constants.COLLECTION_PERMISSIONS)



import codecs
import json
import mock
import os
import pytest
import unittest

from jsonschema.exceptions import ValidationError

from amo2kinto import constants
from amo2kinto.importer import FIELDS, prepare_amo_records, sync_records, main

ADDONS_DATA = {
    "guid": "sqlmoz@facebook.com",
    "blockID": "i454",
    "details": {
        "who": "All Firefox users who have this extension installed.",
        "created": "2013-05-13T09:43:07Z",
        "name": "Mozilla Service Pack (malware)",
        "bug": "https://bugzilla.mozilla.org/show_bug.cgi?id=871610",
        "why": "This extension is malware posing as Mozilla software. "
        "It hijacks Facebook accounts and spams other Facebook users."
    },
    "versionRange": [{
        "targetApplication": [
            {"guid": "{ec8030f7-c20a-464f-9b0e-13a3a9e97384}",
             "minVersion": "3.6",
             "maxVersion": "3.6.*"},
            {"guid": "{some-other-application}",
             "minVersion": "1.2",
             "maxVersion": "1.4"}
        ],
        "minVersion": "0",
        "maxVersion": "*",
        "severity": 3
    }, {
        "targetApplication": [],
        "minVersion": "0",
        "maxVersion": "*",
        "severity": 3
    }],
    "prefs": ["test.blocklist"]
}


def test_addon_record():
    assert prepare_amo_records([ADDONS_DATA], FIELDS['addons']) == [{
        "id": "e3e8f123-588d-0f73-63d8-93bdfc6ae8e2",
        "guid": "sqlmoz@facebook.com",
        "blockID": "i454",
        "details": {
            "who": "All Firefox users who have this extension installed.",
            "created": "2013-05-13T09:43:07Z",
            "name": "Mozilla Service Pack (malware)",
            "bug": "https://bugzilla.mozilla.org/show_bug.cgi?id=871610",
            "why": "This extension is malware posing as Mozilla software. "
            "It hijacks Facebook accounts and spams other Facebook users."
        },
        "versionRange": [{
            'targetApplication': [
                {'guid': '{ec8030f7-c20a-464f-9b0e-13a3a9e97384}',
                 'minVersion': '3.6',
                 'maxVersion': '3.6.*'},
                {'guid': '{some-other-application}',
                 'minVersion': '1.2',
                 'maxVersion': '1.4'}
            ],
            "minVersion": "0",
            "maxVersion": "*",
            "severity": 3
        }, {
            "targetApplication": [],
            "minVersion": "0",
            "maxVersion": "*",
            "severity": 3
        }],
        "prefs": ["test.blocklist"]
    }]


PLUGIN_DATA = {
    "blockID": "p26",
    "matchName": "^Yahoo Application State Plugin$",
    "matchFilename": "npYState.dll",
    "matchDescription": "^Yahoo Application State Plugin$",
    "details": {
        "who": "Users of all versions of Yahoo Application State Plugin "
        "for Firefox 3 and later.\r\n\r\nUsers of all versions of Yahoo "
        "Application State Plugin for SeaMonkey 1.0.0.5 and later.",
        "created": "2011-03-31T16:28:26Z",
        "name": "Yahoo Application State Plugin",
        "bug": "https://bugzilla.mozilla.org/show_bug.cgi?id=421993",
        "why": "This plugin causes a high volume of Firefox and SeaMonkey "
        "crashes."
    },
    "versionRange": [{
        "minVersion": "0",
        "maxVersion": "4.1.10328.0",
        "severity": 0,
        "vulnerabilityStatus": 1,
        "targetApplication": [{
            "minVersion": "3.0a1",
            "guid": "{ec8030f7-c20a-464f-9b0e-13a3a9e97384}",
            "maxVersion": "3.*"
        }]
    }]
}


def test_plugin_record():
    assert prepare_amo_records([PLUGIN_DATA], FIELDS['plugins']) == [{
        "id": "6a1b6dfe-f463-3061-e8f8-6e896ccf2a8a",
        "blockID": "p26",
        "matchName": "^Yahoo Application State Plugin$",
        "matchFilename": "npYState.dll",
        "matchDescription": "^Yahoo Application State Plugin$",
        "details": {
            "who": "Users of all versions of Yahoo Application State Plugin "
            "for Firefox 3 and later.\r\n\r\nUsers of all versions of Yahoo "
            "Application State Plugin for SeaMonkey 1.0.0.5 and later.",
            "created": "2011-03-31T16:28:26Z",
            "name": "Yahoo Application State Plugin",
            "bug": "https://bugzilla.mozilla.org/show_bug.cgi?id=421993",
            "why": "This plugin causes a high volume of Firefox and SeaMonkey "
            "crashes."
        },
        "versionRange": [{
            "minVersion": "0",
            "maxVersion": "4.1.10328.0",
            "severity": 0,
            "vulnerabilityStatus": 1,
            "targetApplication": [{
                "minVersion": "3.0a1",
                "guid": "{ec8030f7-c20a-464f-9b0e-13a3a9e97384}",
                "maxVersion": "3.*"
            }]
        }]
    }]


GFX_DATA = {
    "blockID": "g35",
    "devices": ["0x0a6c"],
    "driverVersion": "8.17.12.5896",
    "driverVersionComparator": "LESS_THAN_OR_EQUAL",
    "feature": "DIRECT2D",
    "featureStatus": "BLOCKED_DRIVER_VERSION",
    "vendor": "0x10de",
    "details": {
        "who": "All Firefox users who have these drivers installed.",
        "created": "2012-09-24T08:23:33Z",
        "name": "ATI/AMD driver 8.982.0.0",
        "bug": "https://bugzilla.mozilla.org/show_bug.cgi?id=792480",
        "why": "Some features in these drivers are causing frequent crashes "
        "in Firefox."
    },
    "os": "WINNT 6.1"
}


def test_gfx_record():
    assert prepare_amo_records([GFX_DATA], FIELDS['gfx']) == [{
        "id": "00a6b9d2-285f-83f0-0a1f-ef0205a60067",
        "blockID": "g35",
        "devices": ["0x0a6c"],
        "driverVersion": "8.17.12.5896",
        "driverVersionComparator": "LESS_THAN_OR_EQUAL",
        "feature": "DIRECT2D",
        "featureStatus": "BLOCKED_DRIVER_VERSION",
        "vendor": "0x10de",
        "details": {
            "who": "All Firefox users who have these drivers installed.",
            "created": "2012-09-24T08:23:33Z",
            "name": "ATI/AMD driver 8.982.0.0",
            "bug": "https://bugzilla.mozilla.org/show_bug.cgi?id=792480",
            "why": "Some features in these drivers are causing frequent "
            "crashes in Firefox."
        },
        "os": "WINNT 6.1"
    }]


CERTIFICATE_DATA = {
    "blockID": "c796",
    "issuerName": "MBQxEjAQBgNVBAMTCWVEZWxsUm9vdA==",
    "serialNumber": "a8V7lRiTqpdLYkrAiPw7tg==",
    "details": {
        "who": ".",
        "created": "2015-11-24T14:40:34Z",
        "name": "eDellRoot",
        "bug": "https://bugzilla.mozilla.org/show_bug.cgi?id=1227729",
        "why": "."
    }
}


def test_certificate_record():
    assert prepare_amo_records([CERTIFICATE_DATA], FIELDS['certificates']) == [
        {'id': 'fe7681eb-8480-718e-9870-084dca698f1d',
         'issuerName': 'MBQxEjAQBgNVBAMTCWVEZWxsUm9vdA==',
         'serialNumber': 'a8V7lRiTqpdLYkrAiPw7tg==',
         "details": {
             "who": ".",
             "created": "2015-11-24T14:40:34Z",
             "name": "eDellRoot",
             "bug": "https://bugzilla.mozilla.org/show_bug.cgi?id=1227729",
             "why": "."}}]


def test_sync_records_calls_the_scenario():
    prepared_records = []
    with mock.patch(
            'amo2kinto.importer.get_kinto_records',
            return_value=mock.sentinel.kinto_records) as get_kinto_records:
        with mock.patch(
                'amo2kinto.importer.prepare_amo_records',
                return_value=prepared_records) as p_amo_records:
            with mock.patch(
                    'amo2kinto.importer.get_diff',
                    return_value=(
                        mock.sentinel.to_create,
                        mock.sentinel.to_update,
                        mock.sentinel.to_delete)) as get_diff:
                with mock.patch(
                        'amo2kinto.importer.push_changes') as push_changes:

                    amo_records = []

                    sync_records(amo_records,
                                 mock.sentinel.fields,
                                 mock.sentinel.kinto_client,
                                 mock.sentinel.bucket,
                                 mock.sentinel.collection,
                                 mock.sentinel.config,
                                 mock.sentinel.permissions)

                    p_amo_records.assert_called_with(
                        amo_records,
                        mock.sentinel.fields)

                    get_kinto_records.assert_called_with(
                        kinto_client=mock.sentinel.kinto_client,
                        bucket=mock.sentinel.bucket,
                        collection=mock.sentinel.collection,
                        config=mock.sentinel.config,
                        permissions=mock.sentinel.permissions)

                    get_diff.assert_called_with(
                        prepared_records,
                        mock.sentinel.kinto_records)

                    push_changes.assert_called_with(
                        (mock.sentinel.to_create,
                         mock.sentinel.to_update,
                         mock.sentinel.to_delete),
                        mock.sentinel.kinto_client,
                        bucket=mock.sentinel.bucket,
                        collection=mock.sentinel.collection)


SCHEMAS = {
    'certificates': None,
    'gfx': None,
    'addons': None,
    'plugins': None,
}


with codecs.open(constants.SCHEMA_FILE, encoding='utf-8') as f:
    SCHEMAS = json.load(f)['collections']


RECORDS = {
    'certificates': None,
    'gfx': None,
    'addons': None,
    'plugins': None,
}

RECORDS_FILE = os.path.join(os.path.dirname(__file__),
                            'fixtures', 'blocklists.json')
with codecs.open(RECORDS_FILE, encoding='utf-8') as f:
    RECORDS = json.load(f)


class TestSyncRecords(unittest.TestCase):
    def setUp(self):
        p = mock.patch('amo2kinto.importer.get_kinto_records', return_value=[])
        self.addCleanup(p.stop)
        p.start()
        p = mock.patch('amo2kinto.importer.push_changes')
        self.addCleanup(p.stop)
        p.start()

    def test_sync_records_validate_records_against_schema(self):
        # sync_records should not raise an Exception here.
        sync_records(RECORDS['addons'],
                     FIELDS['addons'],
                     mock.sentinel.kinto_client,
                     mock.sentinel.bucket,
                     mock.sentinel.collection,
                     SCHEMAS['addons']['config'],
                     mock.sentinel.permissions)

    def test_sync_records_fails_if_the_schema_does_not_validate_records(self):
        # Make sure it raises an exception with wrong records.
        with pytest.raises(ValidationError):
            sync_records(RECORDS['gfx'],
                         FIELDS['gfx'],
                         mock.sentinel.kinto_client,
                         mock.sentinel.bucket,
                         mock.sentinel.collection,
                         SCHEMAS['addons']['config'],
                         mock.sentinel.permissions)


class TestMain(unittest.TestCase):
    def setUp(self):
        # Mock requests
        request_mock = mock.MagicMock()
        request_mock.json.return_value = RECORDS
        p = mock.patch('requests.get', return_value=request_mock)
        self.addCleanup(p.stop)
        p.start()

        # Mock client
        self.MockedClient = mock.MagicMock()
        self.MockedClient.return_value.session.request.return_value = {
            'capabilities': {}
        }, {}
        p = mock.patch('kinto_client.cli_utils.Client', self.MockedClient)
        self.addCleanup(p.stop)
        p.start()

    def assert_arguments(self, mock_sync, kinto_client, **kwargs):
        kwargs.setdefault('kinto_server', constants.KINTO_SERVER)
        kwargs.setdefault('auth', constants.AUTH)
        kwargs.setdefault('certificates_bucket', constants.CERT_BUCKET)
        kwargs.setdefault('certificates_collection', constants.CERT_COLLECTION)
        kwargs.setdefault('gfx_bucket', constants.GFX_BUCKET)
        kwargs.setdefault('gfx_collection', constants.GFX_COLLECTION)
        kwargs.setdefault('addons_bucket', constants.ADDONS_BUCKET)
        kwargs.setdefault('addons_collection', constants.ADDONS_COLLECTION)
        kwargs.setdefault('plugins_bucket', constants.PLUGINS_BUCKET)
        kwargs.setdefault('plugins_collection', constants.PLUGINS_COLLECTION)
        kwargs.setdefault('no_schema', False)
        kwargs.setdefault('amo_server', constants.ADDONS_SERVER)
        kwargs.setdefault('schemas', SCHEMAS)

        kinto_client.assert_called_with(server_url=kwargs['kinto_server'],
                                        auth=kwargs['auth'],
                                        bucket=None,
                                        collection=None)

        cert_config = kwargs['schemas']['certificates']['config']
        gfx_config = kwargs['schemas']['gfx']['config']
        addons_config = kwargs['schemas']['addons']['config']
        plugins_config = kwargs['schemas']['plugins']['config']

        if kwargs['no_schema']:
            cert_config = None
            gfx_config = None
            addons_config = None
            plugins_config = None

        cert_arguments = {
            'amo_records': RECORDS['certificates'],
            'fields': FIELDS['certificates'],
            'kinto_client': kinto_client.return_value,
            'bucket': kwargs['certificates_bucket'],
            'collection': kwargs['certificates_collection'],
            'config': cert_config,
            'permissions': constants.COLLECTION_PERMISSIONS,
        }

        mock_sync.assert_any_call(**cert_arguments)

        gfx_arguments = {
            'amo_records': RECORDS['gfx'],
            'fields': FIELDS['gfx'],
            'kinto_client': kinto_client.return_value,
            'bucket': kwargs['gfx_bucket'],
            'collection': kwargs['gfx_collection'],
            'config': gfx_config,
            'permissions': constants.COLLECTION_PERMISSIONS,
        }

        mock_sync.assert_any_call(**gfx_arguments)

        addons_arguments = {
            'amo_records': RECORDS['addons'],
            'fields': FIELDS['addons'],
            'kinto_client': kinto_client.return_value,
            'bucket': kwargs['addons_bucket'],
            'collection': kwargs['addons_collection'],
            'config': addons_config,
            'permissions': constants.COLLECTION_PERMISSIONS,
        }

        mock_sync.assert_any_call(**addons_arguments)

        plugins_arguments = {
            'amo_records': RECORDS['plugins'],
            'fields': FIELDS['plugins'],
            'kinto_client': kinto_client.return_value,
            'bucket': kwargs['plugins_bucket'],
            'collection': kwargs['plugins_collection'],
            'config': plugins_config,
            'permissions': constants.COLLECTION_PERMISSIONS,
        }

        mock_sync.assert_any_call(**plugins_arguments)

    def test_main_default(self):
        # let's check that main() parsing uses our defaults
        with mock.patch('amo2kinto.importer.sync_records') as mock_sync:
            main([])

        self.assert_arguments(mock_sync, self.MockedClient)

    def test_warning_on_server_schema_capability_missing(self):
        with mock.patch('amo2kinto.importer.sync_records'):
            with mock.patch('amo2kinto.importer.logger') as mocked_logger:
                # capabilities is mocked to an empty value dict in the setUp.
                main([])

        self.assertEqual(mocked_logger.warn.call_count, 2)

    def test_no_warning_on_server_schema_capability_enabled(self):
        with mock.patch('amo2kinto.importer.sync_records'):
            with mock.patch('amo2kinto.importer.logger') as mocked_logger:
                self.MockedClient.return_value.session.request.return_value = {
                    'capabilities': {"schema": {}}
                }, {}
                main([])

        self.assertEqual(mocked_logger.warn.call_count, 0)

    def test_no_schema_option_does_add_the_schema(self):
        with mock.patch('amo2kinto.importer.sync_records') as mock_sync:
            main(['--no-schema'])

        self.assert_arguments(mock_sync, self.MockedClient,
                              no_schema=True)

    def test_main_custom_server(self):
        with mock.patch('amo2kinto.importer.sync_records') as mock_sync:
            main(['-s', 'http://yeah'])

        self.assert_arguments(mock_sync, self.MockedClient,
                              kinto_server='http://yeah')

    def test_can_define_the_certificates_bucket_and_collection(self):
        with mock.patch('amo2kinto.importer.sync_records') as mock_sync:
            main(['--certificates-bucket', 'bucket',
                  '--certificates-collection', 'collection'])

        self.assert_arguments(mock_sync, self.MockedClient,
                              certificates_bucket='bucket',
                              certificates_collection='collection')

    def test_can_define_the_gfx_bucket_and_collection(self):
        with mock.patch('amo2kinto.importer.sync_records') as mock_sync:
            main(
                ['--gfx-bucket', 'bucket',
                 '--gfx-collection', 'collection'])
        self.assert_arguments(mock_sync, self.MockedClient,
                              gfx_bucket='bucket',
                              gfx_collection='collection')

    def test_can_define_the_addons_bucket_and_collection(self):
        with mock.patch('amo2kinto.importer.sync_records') as mock_sync:
            main(
                ['--addons-bucket', 'bucket',
                 '--addons-collection', 'collection'])
        self.assert_arguments(mock_sync, self.MockedClient,
                              addons_bucket='bucket',
                              addons_collection='collection')

    def test_can_define_the_plugins_bucket_and_collection(self):
        with mock.patch('amo2kinto.importer.sync_records') as mock_sync:
            main(
                ['--plugins-bucket', 'bucket',
                 '--plugins-collection', 'collection'])
        self.assert_arguments(mock_sync, self.MockedClient,
                              plugins_bucket='bucket',
                              plugins_collection='collection')

    def test_can_define_the_auth_credentials(self):
        with mock.patch('amo2kinto.importer.sync_records') as mock_sync:
            main(['--auth', 'user:pass'])

        self.assert_arguments(mock_sync, self.MockedClient,
                              auth=('user', 'pass'))

    def test_no_collections_means_all_collections(self):
        """If no 'collection' is passed as parameter, import all of them."""
        with mock.patch('amo2kinto.importer.sync_records') as mock_sync:
            main([])

        # Nothing specific to be tested here, it's the default behavior
        self.assert_arguments(mock_sync, self.MockedClient)

    def test_only_import_certificats(self):
        """If only one collection is specified, only import it."""
        with mock.patch('amo2kinto.importer.sync_records') as mock_sync:
            main(['--certificates', '--no-schema'])

        mock_sync.assert_called_once_with(
            amo_records=RECORDS['certificates'],
            fields=FIELDS['certificates'],
            kinto_client=mock.ANY,
            bucket=constants.CERT_BUCKET,
            collection=constants.CERT_COLLECTION,
            config=None,
            permissions=constants.COLLECTION_PERMISSIONS)

    def test_only_import_certs_and_gfx(self):
        """Only import specified collections"""
        with mock.patch('amo2kinto.importer.sync_records') as mock_sync:
            main(['--certificates', '-G', '--no-schema'])

        assert mock_sync.call_count == 2  # Only called for certificats and gfx
        mock_sync.assert_has_calls([
            mock.call(
                amo_records=RECORDS['certificates'],
                fields=FIELDS['certificates'],
                kinto_client=mock.ANY,
                bucket=constants.CERT_BUCKET,
                collection=constants.CERT_COLLECTION,
                config=None,
                permissions=constants.COLLECTION_PERMISSIONS),
            mock.call(
                amo_records=RECORDS['gfx'],
                fields=FIELDS['gfx'],
                kinto_client=mock.ANY,
                bucket=constants.GFX_BUCKET,
                collection=constants.GFX_COLLECTION,
                config=None,
                permissions=constants.COLLECTION_PERMISSIONS)],
            any_order=True)



from amo2kinto.amo import make_id_from_string, prepare_amo_records


def test_make_id_from_string():
    """It should handle strings, list, int and dict."""
    assert make_id_from_string("foo") == 'acbd18db-4cc2-f85c-edef-654fccc4a4d8'


def test_prepare_amo_records():
    """It should handle strings, list, int, boolean and dict."""
    fields = ['foo', 'bar']
    records = [{
        'blockID': 'c1',
        'foo': 'toto',
        'bar': 'tata',
        'unknown': 'ignore'
    }, {
        'blockID': 'c2',
        'foo': 'taratata',
        'bar': 'foobar',
    }]

    assert prepare_amo_records(records, fields) == [{
        'id': 'a9f7e979-65d6-cf79-9a52-9102a973b8b9',
        'foo': 'toto',
        'bar': 'tata',
    }, {
        'id': '9ab62b5e-f34a-9854-38bf-df7ee0102229',
        'foo': 'taratata',
        'bar': 'foobar',
    }]



import json
import mock
import os
from six import StringIO

from amo2kinto.verifier import sort_lists_in_dict, main


def build_path(filename):
    return os.path.join(os.path.dirname(__file__), 'fixtures', filename)


def test_sort_lists_in_dict_handles_recursion():
    assert json.dumps(sort_lists_in_dict({
        '@name': 'judith',
        'validators': [{
            '@id': 'gbc',
            'toto': ['b', 'a']
        }, {
            '@id': 'abc',
            'toto': ['c', 'd', 'a'],
            'apps': [{
                '@guid': 'cde',
                'minVersion': 2,
            }, {
                '@guid': 'abc',
                'minVersion': 3
            }]
        }]
    }), sort_keys=True) == (
        '{"@name": "judith", "validators": [{'
        '"@id": "abc", '
        '"apps": [{"@guid": "abc", "minVersion": 3}, '
        '{"@guid": "cde", "minVersion": 2}], '
        '"toto": ["a", "c", "d"]}, '
        '{"@id": "gbc", "toto": ["a", "b"]}]}'
    )


def test_files_checking():
    with mock.patch('sys.stdout', new_callable=StringIO) as stdout:
        with mock.patch('sys.stderr', new_callable=StringIO) as stderr:
            main([build_path('blocklist.xml'),
                  build_path('generated-blocklist.xml')])

        assert stdout.getvalue() == ''
        assert stderr.getvalue() == ''


def test_fails_if_file_does_not_exists():
    assert main(['unknown']) == 1


def test_verifier_supports_http_links():
    with open(build_path('blocklist.xml')) as f:
        blocklist_content = f.read()

    response = mock.MagicMock(text=blocklist_content)
    with mock.patch('requests.get', return_value=response) as mocked_request:
        main(['http://first_server/url/', 'http://second_server/url/'])

        mocked_request.assert_any_call('http://first_server/url/')
        mocked_request.assert_any_call('http://second_server/url/')


def test_clean_option_does_not_remove_tmp_files():
    with mock.patch('sys.stderr', new_callable=StringIO) as stderr:
        main([build_path('blocklist.xml'),
              build_path('generated-blocklist.xml'), '-k'])

        content = stderr.getvalue()
        assert content.startswith('$ diff -U10 -u'), content


def test_in_case_diff_fails_display_the_error():
    with mock.patch('sys.stderr', new_callable=StringIO) as stderr:
        main([build_path('fennec-blocklist.xml'),
              build_path('generated-blocklist.xml')])

        assert stderr.getvalue() != ''


def test_in_case_diff_fails_exit_with_2():
    with mock.patch('sys.stderr', new_callable=StringIO):
        assert main([build_path('fennec-blocklist.xml'),
                     build_path('generated-blocklist.xml')]) == 2



import codecs
import os
import sys
from setuptools import setup, find_packages

HERE = os.path.abspath(os.path.dirname(__file__))

with codecs.open(os.path.join(HERE, 'README.rst'), encoding='utf-8') as f:
    README = f.read()

with codecs.open(os.path.join(HERE, 'CHANGELOG.rst'), encoding='utf-8') as f:
    CHANGELOG = f.read()


REQUIREMENTS = [
    'jsonschema',
    'kinto-client',
    'requests',
    'lxml',
    'six',
    'xmltodict',
]


if sys.version_info < (2, 7, 9):
    # Add OpenSSL dependencies to handle requests SSL warning.
    REQUIREMENTS.append([
        "pyopenssl",
        "ndg-httpsclient",
        "pyasn1"
    ])


ENTRY_POINTS = {
    'console_scripts': [
        'kinto2xml = amo2kinto.exporter:main',
        'json2kinto = amo2kinto.importer:main',
        'xml-verifier = amo2kinto.verifier:main',
    ]
}


setup(name='amo2kinto',
      version='1.3.0',
      description='Generate a blocklists.xml file from the Kinto collections.',
      long_description=README + "\n\n" + CHANGELOG,
      license='Apache License (2.0)',
      classifiers=[
          "Programming Language :: Python",
          "Programming Language :: Python :: 2",
          "Programming Language :: Python :: 2.7",
          "Programming Language :: Python :: 3",
          "Programming Language :: Python :: 3.4",
          "License :: OSI Approved :: Apache Software License"
      ],
      keywords="web services",
      author='Mozilla Services',
      author_email='services-dev@mozilla.com',
      url='https://github.com/mozilla-services/amo2kinto',
      packages=find_packages(),
      zip_safe=False,
      install_requires=REQUIREMENTS,
      entry_points=ENTRY_POINTS)



"""Parse and normalize two XML files and then generate a diff of both."""
import argparse
import json
import logging
import os
import requests
import subprocess
import six
import sys
import tempfile
import xmltodict

logger = logging.getLogger('xml-verifier')

RECORD_SORT_FIELDS = (
    '@blockID', '@id', '@issuerName', 'serialNumber',
    '@guid', '@name', '@minVersion', '@maxVersion'
)


def get_unique_id(*fields):
    """Generated a unique sort id string for a given record."""
    def wraps(d):
        if isinstance(d, dict):
            return '-'.join([str(v) for v in (d.get(f) for f in fields)])
        else:
            return d

    return wraps


def sort_lists_in_dict(d):
    if not isinstance(d, dict):
        return d

    for key, value in six.iteritems(d):
        if isinstance(value, list):
            value = sorted(value, key=get_unique_id(*RECORD_SORT_FIELDS))
            value = [sort_lists_in_dict(v) for v in value]
        elif isinstance(value, dict):
            value = sort_lists_in_dict(value)
        elif isinstance(value, six.text_type):
            value = value.strip()

        d[key] = value
    return d


def main(args=None):
    parser = argparse.ArgumentParser(
        description='Parse and normalize two XML files and '
        'generate a diff of both.')

    parser.add_argument('files', metavar='N',
                        help='Compare files, line by lines', nargs='+',)
    parser.add_argument('-k', '--keep-tmp-files', action="store_false",
                        dest='clean', help='Keep normalize temporary files')

    args = parser.parse_args(args=args)

    tmp_files = []

    for filepath in args.files:
        if not filepath.startswith('http') and not os.path.exists(filepath):
            logger.error("%s doesn't exists" % filepath)
            return 1

    last_updated = None

    for filepath in args.files:
        # Normalize XML
        curr_file = tempfile.NamedTemporaryFile("w", delete=False)
        tmp_files.append(curr_file)
        if filepath.startswith('http'):
            resp = requests.get(filepath)
            resp.raise_for_status()
            content = resp.text
        else:
            with open(filepath) as f:
                content = f.read()

        d = xmltodict.parse(content)
        # sort lists of the dict
        sort_lists_in_dict(d)

        # Ignore differences in the lastupdate blocklist tag
        # parameter. So that if they differs we ignore it.
        if not last_updated:
            last_updated = d['blocklist']['@lastupdate']
        else:
            d['blocklist']['@lastupdate'] = last_updated

        json.dump(d, curr_file, indent=4, sort_keys=True)
        curr_file.write('\n')

    # Close and clean files
    for f in tmp_files:
        f.close()

    diff_args = ['diff', '-U10', '-u'] + [tf.name for tf in tmp_files]

    # Print the diff command to stderr if we kept the file for debugging.
    if not args.clean:
        sys.stderr.write('$ %s\n' % ' '.join(diff_args))
    try:
        output = subprocess.check_output(diff_args, stderr=subprocess.STDOUT)
        sys.stdout.write(output.decode('utf-8'))
    except subprocess.CalledProcessError as e:
        sys.stderr.write(e.output.decode('utf-8'))
        return 2

    if args.clean:
        for f in tmp_files:
            os.unlink(f.name)
    else:
        sys.stderr.write('$ %s\n' % ' '.join(diff_args))



import hashlib
import six
import uuid


def make_id_from_string(string):
    hashed = hashlib.md5(string.encode('utf-8'))
    return six.text_type(uuid.UUID(hashed.hexdigest()))


def prepare_amo_records(records, fields):
    for record in records:
        blockID = record['blockID']

        # Remove fields we do not want to keep.
        for field in list(record.keys()):
            if field not in fields:
                del record[field]
            else:
                if isinstance(record[field], six.string_types):
                    record[field] = record[field].strip()

        record['id'] = make_id_from_string(blockID)

    return records  # for convenience
